package com.jiuqi.util;

import java.io.IOException;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.Connection;

public abstract class DBConnect
{
    public static final int SYNC_LEVEL_NO = 0;
    public static final int SYNC_LEVEL_ALL = 1;
    public static final int SYNC_LEVEL_THREAD = 2;
    private int conn_use;
    private int translevel;
    private boolean alreadyInTrans;
    protected Connection connection;
    public boolean disableTransaction;
    private int useSyncLevel;
    private ThreadLocal connections;
    private Object threadMetux;
    
    public DBConnect() {
        this.conn_use = 0;
        this.translevel = 0;
        this.alreadyInTrans = false;
        this.connection = null;
        this.disableTransaction = false;
        this.useSyncLevel = 1;
        this.connections = null;
        this.threadMetux = null;
    }
    
    protected abstract Connection createConnection() throws SQLException;
    
    protected void destroyConnection(final Connection conn) throws SQLException {
        conn.close();
    }
    
    public void setSyncLevel(final int value) {
        this.useSyncLevel = value;
        if (this.useSyncLevel == 2) {
            this.connections = new ThreadLocal();
            this.threadMetux = new Object();
        }
    }
    
    public Connection getConnection() throws SQLException {
        switch (this.useSyncLevel) {
            case 0: {
                return null;
            }
            case 1: {
                return this.getConnection0();
            }
            case 2: {
                InnerConnection innerConn = this.connections.get();
                if (innerConn == null) {
                    synchronized (this.threadMetux) {
                        innerConn = this.connections.get();
                        if (innerConn == null) {
                            innerConn = new InnerConnection();
                            this.connections.set(innerConn);
                        }
                        monitorexit(this.threadMetux);
                    }
                }
                return innerConn.getConnection();
            }
            default: {
                return null;
            }
        }
    }
    
    public void releaseConnection() {
        switch (this.useSyncLevel) {
            case 1: {
                this.releaseConnection0();
                break;
            }
            case 2: {
                final InnerConnection innerConn = this.connections.get();
                if (innerConn != null) {
                    innerConn.releaseConnection();
                    break;
                }
                break;
            }
        }
    }
    
    private synchronized void releaseConnection0() {
        --this.conn_use;
        if (this.conn_use == 0) {
            try {
                this.destroyConnection(this.connection);
            }
            catch (final SQLException ex) {
                ex.printStackTrace();
            }
            finally {
                this.connection = null;
            }
            this.connection = null;
        }
    }
    
    private synchronized Connection getConnection0() throws SQLException {
        if (this.connection == null) {
            try {
                this.connection = this.createConnection();
            }
            catch (final SQLException ex) {
                throw ex;
            }
            catch (final Exception ex2) {
                throw new SQLException(ex2.toString());
            }
        }
        ++this.conn_use;
        return this.connection;
    }
    
    private Connection getConnection2() {
        switch (this.useSyncLevel) {
            case 0: {
                return null;
            }
            case 1: {
                return this.connection;
            }
            case 2: {
                final InnerConnection innerConn = this.connections.get();
                if (innerConn != null) {
                    return innerConn.connectionInner;
                }
                break;
            }
        }
        return null;
    }
    
    public synchronized void beginTrans() throws SQLException {
        if (this.disableTransaction) {
            return;
        }
        ++this.translevel;
        if (this.translevel > 1) {
            return;
        }
        final Connection conn = this.getConnection();
        if (!(this.alreadyInTrans = !conn.getAutoCommit())) {
            conn.setAutoCommit(false);
        }
    }
    
    public synchronized void commitTrans() throws SQLException {
        if (this.disableTransaction) {
            return;
        }
        if (this.translevel <= 0) {
            throw new SQLException("不在事务内部，无法提交");
        }
        if (this.translevel != 1) {
            --this.translevel;
        }
        else {
            try {
                if (!this.alreadyInTrans) {
                    this.getConnection2().commit();
                }
            }
            catch (final SQLException ex) {
                ex.printStackTrace();
                throw ex;
            }
            try {
                --this.translevel;
                if (!this.alreadyInTrans) {
                    this.getConnection2().setAutoCommit(true);
                }
                this.releaseConnection();
            }
            catch (final SQLException ex) {
                ex.printStackTrace();
            }
        }
    }
    
    public synchronized void cancelTrans() throws SQLException {
        if (this.disableTransaction) {
            return;
        }
        if (this.translevel <= 0) {
            throw new SQLException("不在事务内部，无法撤销");
        }
        if (this.translevel != 1) {
            --this.translevel;
        }
        else {
            try {
                if (!this.alreadyInTrans) {
                    this.getConnection2().rollback();
                }
                try {
                    if (!this.alreadyInTrans) {
                        this.getConnection2().setAutoCommit(true);
                    }
                }
                catch (final SQLException ex) {
                    ex.printStackTrace();
                }
            }
            finally {
                --this.translevel;
                this.releaseConnection();
            }
            --this.translevel;
            this.releaseConnection();
        }
    }
    
    public synchronized void setIsAutoCommit(final boolean aIsAutoCommit) {
        this.alreadyInTrans = !aIsAutoCommit;
    }
    
    public static void runSql(final Statement stmt, final String sql) throws SQLException {
        boolean isEmpty = true;
        for (int i = 0; i < sql.length(); ++i) {
            final char ch = sql.charAt(i);
            if (Character.isLetterOrDigit(ch)) {
                isEmpty = false;
                break;
            }
        }
        if (!isEmpty) {
            stmt.execute(sql);
        }
    }
    
    public static boolean endWith(final String line, final char ch) {
        if (line == null) {
            return false;
        }
        int index;
        for (index = line.length() - 1; index >= 0 && Character.isWhitespace(line.charAt(index)); --index) {}
        return index >= 0 && line.charAt(index) == ch;
    }
    
    public static void runScript(final String scriptFile, final ArrayList errors, final Statement stmt) throws IOException, SQLException {
        final FileReader fr = new FileReader(scriptFile);
        final BufferedReader reader = new BufferedReader(fr);
        try {
            final StringBuffer sql = new StringBuffer(1024);
            while (true) {
                final String line = reader.readLine();
                if (line != null && !"GO".equalsIgnoreCase(line) && !endWith(line, ';')) {
                    sql.append(line);
                    sql.append('\n');
                }
                else {
                    if (line == null) {
                        break;
                    }
                    if (endWith(line, ';')) {
                        int index;
                        for (index = line.length() - 1; index >= 0 && Character.isWhitespace(line.charAt(index)); --index) {}
                        sql.append(line.substring(0, index));
                    }
                    final String sqlStmt = sql.toString();
                    sql.setLength(0);
                    if (errors == null) {
                        runSql(stmt, sqlStmt);
                    }
                    else {
                        try {
                            runSql(stmt, sqlStmt);
                        }
                        catch (final Exception ex) {
                            errors.add(ex.getMessage());
                        }
                    }
                }
            }
        }
        finally {
            reader.close();
            fr.close();
        }
        reader.close();
        fr.close();
    }
    
    public void runScript(final String scriptFile, final ArrayList errors) throws IOException, SQLException {
        final Connection conn = this.getConnection();
        try {
            final Statement stmt = conn.createStatement(1003, 1007);
            runScript(scriptFile, errors, stmt);
        }
        finally {
            this.releaseConnection();
        }
        this.releaseConnection();
    }
    
    public static boolean isUniqueViolationException(final SQLException ex) {
        String msg = ex.getMessage();
        if (msg == null) {
            return false;
        }
        msg = msg.toLowerCase();
        return (msg.indexOf("unique") >= 0 || msg.indexOf("primary key") >= 0 || msg.indexOf("唯一") >= 0 || msg.indexOf("主键") >= 0) && (msg.indexOf("violate") >= 0 || msg.indexOf("违反") >= 0);
    }
    
    class InnerConnection
    {
        int connUser;
        Connection connectionInner;
        
        InnerConnection() {
            this.connUser = 0;
            this.connectionInner = null;
        }
        
        public synchronized Connection getConnection() throws SQLException {
            if (this.connectionInner == null) {
                try {
                    this.connectionInner = DBConnect.this.createConnection();
                }
                catch (final SQLException ex) {
                    throw ex;
                }
                catch (final Exception ex2) {
                    throw new SQLException(ex2.toString());
                }
            }
            ++this.connUser;
            return this.connectionInner;
        }
        
        public synchronized void releaseConnection() {
            --this.connUser;
            if (this.connUser == 0) {
                try {
                    DBConnect.this.destroyConnection(this.connectionInner);
                }
                catch (final SQLException ex) {
                    ex.printStackTrace();
                }
                finally {
                    this.connectionInner = null;
                }
                this.connectionInner = null;
            }
        }
    }
}
